import os  # pathlib.Path.walk not available in Python <3.12
import matplotlib.pyplot as plt

import scienceplots
from .styles_discovery import read_styles_in_folders

# register the bundled stylesheets in the matplotlib style library
scienceplots_path = scienceplots.__path__[0]
styles_path = os.path.join(scienceplots_path, "styles")

# Reads styles in /styles folder and all subfolders
stylesheets = read_styles_in_folders(styles_path)

# Update dictionary of styles - plt.style.library
plt.style.core.update_nested_dict(plt.style.library, stylesheets)
# Update `plt.style.available`, copy-paste from:
# https://github.com/matplotlib/matplotlib/blob/a170539a421623bb2967a45a24bb7926e2feb542/lib/matplotlib/style/core.py#L266  # noqa: E501
plt.style.core.available[:] = sorted(plt.style.library.keys())



import os

import matplotlib.pyplot as plt


def read_styles_in_folders(root_path):
    """
    Reads all stylesheets in the given path and its subfolders.

    Parameters
    ----------
    root_path : str
        Path to the root folder containing the stylesheets and other subfolders
        with stylesheets.

    Returns
    -------
    stylesheets : dict
        Dictionary of stylesheets in the form of {style_name: rcParams}.
        Should be compatible with matplotlib's plt.style.library dictionary.
    """
    stylesheets = {}  # plt.style.library is a dictionary
    for folder, _, _ in os.walk(root_path):
        new_stylesheets = plt.style.core.read_style_directory(folder)
        stylesheets.update(new_stylesheets)
    return stylesheets



"""Test suite of SciencePlots
"""

import matplotlib.pyplot as plt


def test_matplotlib_required_api_existence():
    """Check if all functions and attributes used by scienceplots are available
    in matplotlib.
    """
    assert hasattr(plt.style, "core")
    assert hasattr(plt.style.core, "read_style_directory")
    assert hasattr(plt.style.core, "update_nested_dict")
    assert hasattr(plt.style, "available")
    assert hasattr(plt.style, "library")


def test_styles_existence(styles_in_scienceplots_per_folder):
    """Check all styles are available in matplotlib."""
    for folder, styles in styles_in_scienceplots_per_folder.items():
        assert len(styles) > 0, f"No styles found in {folder}."
        for style in styles:
            assert (  # both in list of names and the library they are retrieved from hello
                style in plt.style.available and style in plt.style.library
            ), f"'{style}' not in available styles. Style in folder {folder}."


def test_usage_of_each_style(
    xy_example_values, styles_in_scienceplots_per_folder, tmp_path,
):
    """Tests if the styles are correctly formatted and can be applied to a plot."""
    pparam = {"xlabel": "Voltage (mV)", "ylabel": r"Current ($\mu$A)"}
    x, ys, p = xy_example_values
    for folder, styles in styles_in_scienceplots_per_folder.items():
        for style in styles:
            output_file = tmp_path / f"test_{folder}_{style}.png"
            with plt.style.context(style):
                fig, ax = plt.subplots()
                for y in ys:
                    ax.plot(x, y)
                ax.legend(p, title="Order")
                ax.set(**pparam)
                ax.autoscale(tight=True)
                fig.savefig(output_file)
                plt.close(fig)



"""Plot examples of SciencePlot styles."""

# %%
import numpy as np
import matplotlib.pyplot as plt
import scienceplots  # noqa: F401

import os

# Check we are in examples dir
current_dir = os.getcwd().lower()
if current_dir.endswith("scienceplots"):
    os.chdir("./examples")
# Create 'figures' folder if it does not exist
if not os.path.exists("./figures"):
    os.makedirs("figures")


def model(x, p):
    return x ** (2 * p + 1) / (1 + x ** (2 * p))


pparam = dict(xlabel="Voltage (mV)", ylabel=r"Current ($\mu$A)")

x = np.linspace(0.75, 1.25, 201)


# %%
# Style 'science'

with plt.style.context(["science"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig01a.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'no-latex'
with plt.style.context(["science", "no-latex"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig01b.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'ieee'
with plt.style.context(["science", "ieee"]):
    fig, ax = plt.subplots()
    for p in [10, 20, 40, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig02a.jpg", dpi=300)
    plt.close()


# %%
# Styles 'science', 'ieee', 'std-colors'
with plt.style.context(["science", "ieee", "std-colors"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig02b.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'nature'
with plt.style.context(["science", "nature"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig02c.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'scatter'
with plt.style.context(["science", "scatter"]):
    fig, ax = plt.subplots(figsize=(4, 4))
    ax.plot([-2, 2], [-2, 2], "k--")
    ax.fill_between(
        [-2, 2], [-2.2, 1.8], [-1.8, 2.2], color="dodgerblue", alpha=0.2, lw=0
    )
    for i in range(7):
        x1 = np.random.normal(0, 0.5, 10)
        y1 = x1 + np.random.normal(0, 0.2, 10)
        ax.plot(x1, y1, label=r"$^\#${}".format(i + 1))
    lgd = r"$\mathring{P}=\begin{cases}1&\text{if $\nu\geq0$}\\0&\text{if $\nu<0$}\end{cases}$"
    ax.legend(title=lgd, loc=2, ncol=2)
    xlbl = r"$\log_{10}\left(\frac{L_\mathrm{IR}}{\mathrm{L}_\odot}\right)$"
    ylbl = r"$\log_{10}\left(\frac{L_\circledast}{\mathrm{L}_\odot}\right)$"
    ax.set_xlabel(xlbl)
    ax.set_ylabel(ylbl)
    ax.set_xlim([-2, 2])
    ax.set_ylim([-2, 2])
    fig.savefig("figures/fig03.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'high-vis'
with plt.style.context(["science", "high-vis"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig04.jpg", dpi=300)
    plt.close()

# %%
# Styles 'dark_background', 'science', 'high-vis'
with plt.style.context(["dark_background", "science", "high-vis"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig05.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'notebook'
with plt.style.context(["science", "notebook"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig10.jpg", dpi=300)
    plt.close()

# %%
# Plot different color cycles

# %%
# Styles 'science', 'bright'
with plt.style.context(["science", "bright"]):
    fig, ax = plt.subplots()
    for p in [5, 10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig06.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'vibrant'
with plt.style.context(["science", "vibrant"]):
    fig, ax = plt.subplots()
    for p in [5, 10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig07.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'muted'
with plt.style.context(["science", "muted"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100, 500]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order", fontsize=7)
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig08.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'retro'
with plt.style.context(["science", "retro"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig09.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'grid'
with plt.style.context(["science", "grid"]):
    fig, ax = plt.subplots()
    for p in [10, 15, 20, 30, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig11.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'high-contrast'
with plt.style.context(["science", "high-contrast"]):
    fig, ax = plt.subplots()
    for p in [10, 20, 50]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order")
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig12.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'light'
with plt.style.context(["science", "light"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order", fontsize=7)
    ax.autoscale(tight=True)
    ax.set(**pparam)
    fig.savefig("figures/fig13.jpg", dpi=300)
    plt.close()

# %%
# Styles for CJK languages
# Note: You need to install the Noto Serif CJK Fonts before running
# examples 14 and 15. See FAQ in README.

# %%
# Styles 'science', 'no-latex', 'cjk-*-font'
with plt.style.context(["science", "no-latex", "cjk-tc-font"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order", fontsize=7)
    ax.set(xlabel=r"電壓 (mV)")
    ax.set(ylabel=r"電流 ($\mu$A)")
    ax.autoscale(tight=True)
    fig.savefig("figures/fig14a.jpg", dpi=300)
    plt.close()

with plt.style.context(["science", "no-latex", "cjk-sc-font"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order", fontsize=7)
    ax.set(xlabel=r"电压 (mV)")
    ax.set(ylabel=r"电流 ($\mu$A)")
    ax.autoscale(tight=True)
    fig.savefig("figures/fig14b.jpg", dpi=300)
    plt.close()

with plt.style.context(["science", "no-latex", "cjk-jp-font"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order", fontsize=7)
    ax.set(xlabel=r"電圧 (mV)")
    ax.set(ylabel=r"電気 ($\mu$A)")
    ax.autoscale(tight=True)
    fig.savefig("figures/fig14c.jpg", dpi=300)
    plt.close()

with plt.style.context(["science", "no-latex", "cjk-kr-font"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title="Order", fontsize=7)
    ax.set(xlabel=r"전압 (mV)")
    ax.set(ylabel=r"전류 ($\mu$A)")
    ax.autoscale(tight=True)
    fig.savefig("figures/fig14d.jpg", dpi=300)
    plt.close()

# import matplotlib
# matplotlib.use('pgf')  # stwich backend to pgf
# matplotlib.rcParams.update({
#     "pgf.preamble": [
#         "\\usepackage{fontspec}",
#         '\\usepackage{xeCJK}',
#         r'\setmainfont{Times New Roman}',  # EN fonts Romans
#         r'\setCJKmainfont{SimHei}',  # set CJK fonts as SimSun
#         r'\setCJKsansfont{SimHei}',
#         r'\newCJKfontfamily{\Song}{SimSun}',
#         ]
# })

# with plt.style.context(['science', 'cjk-tc-font']):
#     fig, ax = plt.subplots()
#     for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
#         ax.plot(x, model(x, p), label=p)
#     ax.legend(title='Order', fontsize=7)
#     ax.set(xlabel=r'電壓 (mV)')
#     ax.set(ylabel=r'電流 ($\mu$A)')
#     ax.autoscale(tight=True)
#     fig.savefig('figures/fig15.pdf', backend='pgf')
#     plt.close()

# %%
# Styles 'science', 'russian-font'
with plt.style.context(["science", "russian-font"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title=r"Число", fontsize=7)
    ax.set(xlabel=r"Напряжение (mV)")
    ax.set(ylabel=r"Сила тока ($\mu$A)")
    ax.autoscale(tight=True)
    fig.savefig("figures/fig16.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'turkish-font'
with plt.style.context(["science", "turkish-font"]):
    fig, ax = plt.subplots()
    for p in [5, 7, 10, 15, 20, 30, 38, 50, 100]:
        ax.plot(x, model(x, p), label=p)
    ax.legend(title=r"Düzen", fontsize=7)
    ax.set(xlabel=r"Gerilim/Volt (mV)")
    ax.set(ylabel=r"Mevcut Güç/Akım ($\mu$A)")
    ax.autoscale(tight=True)
    fig.savefig("figures/fig17.jpg", dpi=300)
    plt.close()

# %%
# Styles 'science', 'no-latex', 'discrete-rainbow-*'
# Plot all discrete-rainbow styles
p_list = np.logspace(0, 2, 23)
for index in range(1, 23 + 1):
    style_name = f"discrete-rainbow-{index}"
    figure_name = f"fig_dr_{index}.jpg"
    with plt.style.context(["science", "no-latex", style_name]):
        fig, ax = plt.subplots()
        for p in p_list[: index]:
            ax.plot(x, model(x, p))
        ax.autoscale(tight=True)
        ax.set(**pparam)
        fig.savefig(f"figures/{figure_name}", dpi=300)
        plt.close()

# %%



"""
Configuration of SciencePlots tests
"""

import pytest
import scienceplots
import numpy as np

import os
from pathlib import Path

STYLES_PATH = Path(scienceplots.__path__[0], "styles")


def get_styles_in_dir(dir):
    """
    Input: directory path
    Output: set of matplotlib styles filenames (without trailing '.mplstyle')
    """
    styles_paths = Path(dir).glob("*.mplstyle")
    return set(fn.stem for fn in styles_paths)


@pytest.fixture(scope="session")
def styles_in_scienceplots_per_folder():
    """
    Output: dictionary of styles per folder in SciencePlots
    """
    styles_per_folder = {}
    for folder, _, _ in os.walk(STYLES_PATH):  # 1st _ : subdirs, 2nd _ : files
        folder = Path(folder)
        styles_per_folder[folder.name] = get_styles_in_dir(folder)
    return styles_per_folder


@pytest.fixture(scope="session")
def xy_example_values():
    """
    Output: "x" 1D values, "y" 2D values, and 1D "p" values for the example plot
    """

    def model(x, p):  # from examples/plot-examples.py
        return x ** (2 * p + 1) / (1 + x ** (2 * p))

    x = np.linspace(0.75, 1.25, 201)
    p = [10, 15, 20, 30, 50, 100]
    res = np.fromiter(map(lambda p: model(x, p), p), dtype=np.dtype((float, len(x))))
    return x, res, p
